#
# Copyright (c) 2023, HILSTER - https://hilster.io
# All rights reserved.
#

from typing import Callable, Any

import functools


def locked(func: Callable[..., Any]) -> Callable[..., Any]:
    @functools.wraps(func)
    def _wrapper(self: Any, *args: Any, **kwargs: Any) -> Any:
        with self._lock:  # type: ignore
            return func(self, *args, **kwargs)

    return _wrapper


def transactional(func: Callable[..., Any]) -> Callable[..., Any]:
    @functools.wraps(func)
    def _wrapper(self: Any, *args: Any, **kwargs: Any) -> Any:
        with self._transaction_lock:  # type: ignore
            return func(self, *args, **kwargs)

    return _wrapper
